"""
Streamlit Application
Marie FU
"""

import re
import os
import glob
import shutil
import tempfile
from zipfile import ZipFile
import streamlit as st
import src.basic as tmp
from bs4 import BeautifulSoup
from grobid_client.grobid_client import GrobidClient, ServerUnavailableException
from src.gerneralFileOP import *

STATE = st.session_state

def empty_content() -> None:
    for f in glob.glob("./book/file*"):
        os.remove(f)


def getXMLContent(filePath : str) -> bool:

    """
    Get given file content

    Args:
        filePath (String) : absolute path of file

    Returns:
        None

    Authors:
        Marie FU
    """

    tei_doc = filePath
    try:
        with open(tei_doc, 'r') as tei:
            STATE.soup = BeautifulSoup(tei, 'xml')
        return True
    except FileNotFoundError:
        st.write(st.session_state.general_text_dict["err_grobid"])
        empty_content()
        return False


def getBookTitle() -> None:
        
    """
    Get the title of the PDF book

    Args:
        None

    Returns:
        None

    Authors:
        Marie FU
    """
        
    STATE.title = ""
    if STATE.soup.title is not None:
        STATE.title = STATE.soup.title.string
    else:
        STATE.title = None
    return


def getDate() -> None:
        
    """
    Get the publication date of the PDF book

    Args:
        None

    Returns:
        None

    Authors:
        Marie FU
    """
        
    STATE.date = ""
    date_ = STATE.soup.find('date')
    if date_ is not None:
        try:
            STATE.date = date_['when']
        except KeyError:
            STATE.date = "1\t1\t1"
    if "-" in STATE.date:
        STATE.date = STATE.date.replace("-","\t")
    return


def getAuthors() -> None:
        
    """
    Get the authors of the PDF book

    Args:
        None

    Returns:
        None

    Authors:
        Marie FU
    """
        
    STATE.authors = ""
    authorList = []
    if STATE.soup.sourceDesc is not None:
        authorList = STATE.soup.sourceDesc.findAll("author")

    for author in authorList:
        if author is not None:
            if STATE.authors != "" and (author.forename is not None or author.surname is not None):
                STATE.authors += "; "
            forename_list = author.findAll("forename")
            forename_ = ""
            for forename in forename_list:
                if forename is not None:
                    if forename_ != "":
                        forename_ += " "
                    forename_ += forename.string
            STATE.authors += forename_
            surname_list = author.findAll("surname")
            surname_ = ""
            for surname in surname_list:
                if surname is not None:
                    if surname_ != "":
                        surname_ += " "
                    surname_ += surname.string
            STATE.authors += " " + surname_
            if author.roleName is not None and (author.forename is not None or author.surname is not None):
                STATE.authors += ", " + author.roleName.string
    if len(STATE.authors) == 0:
        STATE.authors = "Unknown authors"
    STATE.authors = STATE.authors.strip()
    return


def getDataPerParagraph() -> None:
        
    """
    Get parsed content of the PDF book per paragraph

    Args:
        None

    Returns:
        None

    Authors:
        Marie FU
    """
        
    STATE.fileData = ""
    if STATE.soup.body is not None:
        divList = STATE.soup.body.find_all('div')
        
        # Loop in sections
        for div in divList:
            STATE.source = ""
            if div is not None:
                paragraphList = div.findAll('p')

                if div.head is not None:
                    STATE.source = div.head.string
                else:
                    STATE.source = None

            # Loop in paragraphs
            for paragraph in paragraphList:
                STATE.fileData += STATE.date + "\t"
                if STATE.source is not None and STATE.title is not None:
                    STATE.fileData += STATE.title + ", " + STATE.source + "\t"
                elif STATE.source is None and STATE.title is not None:
                    STATE.fileData += STATE.title + "\t"
                elif STATE.source is not None and STATE.title is None:
                    STATE.fileData += "Unknown Book, " + STATE.source + "\t"
                else:
                    STATE.fileData += "Unknown Book\t"

                if paragraph is not None:
                    sentenceList = paragraph.findAll('s')

                # First sentence of paragraph checker
                first = True
                # Loop in sentences
                for sentence in sentenceList:
                    if sentence is not None and len(sentence.get_text()) != 0:
                        if first:
                            STATE.fileData += sentence.get_text() + "\t" + sentence.get_text()
                            first = False
                        else:
                            STATE.fileData += " " + sentence.get_text()
                if first:
                    first = False
                    STATE.fileData += "Title\tAbstract"
                STATE.fileData += "\n"

    STATE.fileData = "Publication Year\tPublication Month\tPublication Day\tSource\tTitle\tAbstract\tAuthors\n" + STATE.fileData.replace("\n","\t" + STATE.authors+"\n")
    return


def getFile() -> None:

    """
    Get the TSV file after submiting the form concerning book information

    Args:
        None

    Returns:
        None

    Authors:
        Marie FU
    """

    dateTab = STATE.date.split("\t")
    yearVal : int = 1
    monthVal : int = 1
    dayVal : int = 1
    if len(dateTab) == 3 :
        yearVal = int(dateTab[0])
        monthVal = int(dateTab[1])
        dayVal = int(dateTab[1])
    elif len(dateTab) == 2 :
        yearVal = int(dateTab[0])
        monthVal = int(dateTab[1])
        dayVal = 1
    elif len(dateTab) == 1 :
        yearVal = int(dateTab[0])
        monthVal = 1
        dayVal = 1

    placeholder = st.empty()
    with placeholder.form(key="infoDoc",clear_on_submit=True):
        st.write(st.session_state.general_text_dict['pub_date'])
        col1, col2, col3 = st.columns(3)
        with col1:
            year_ = st.number_input(label=st.session_state.general_text_dict['date_year'],min_value=1,value=yearVal,step=1)
        with col2:
            month_ = st.number_input(label=st.session_state.general_text_dict['date_month'],min_value=1,max_value=12,value=monthVal,step=1)
        with col3:
            day_ = st.number_input(label=st.session_state.general_text_dict['date_day'],min_value=1,max_value=31,value=dayVal,step=1)
        st.write(st.session_state.general_text_dict['book_title'])
        title_ = st.text_input(label=st.session_state.general_text_dict['book_title'],value=STATE.title,label_visibility="collapsed")
        st.write(st.session_state.general_text_dict['authors'])
        authors_ = st.text_input(label=st.session_state.general_text_dict['authors'],value=STATE.authors,label_visibility="collapsed")
        submit = st.form_submit_button()
    if submit:
        placeholder.empty()
        if title_ :
            STATE.title = title_
        if authors_ :
            STATE.authors = authors_
        STATE.date = f"{year_}\t{month_}\t{day_}"
        STATE.noData = False
        empty_content()

        if STATE.sepBy == 0:
            getDataPerParagraph()
        else:
            getDataPerNSentences()

        print(STATE.newFileName)
        show_download_button(STATE.newFileName,STATE.fileData.replace("\"","”"))
        return


def requestGC(dir) -> None:

    """
    Create the GROBID client

    Args:
        None

    Returns:
        None

    Authors:
        Marie FU
    """
    
    STATE.client.process(service="processFulltextDocument", input_path=dir, output=dir, segment_sentences=True, consolidate_header=True)
    print("request finished")


def getDataPerNSentences() -> None:
        
    """
    Get parsed content of the PDF book per n sentences

    Args:
        None

    Returns:
        None

    Authors:
        Marie FU
    """
        
    STATE.fileData = ""
    if STATE.soup.body is not None:
        sentenceList = STATE.soup.body.find_all('s')

        nb = 0
        # Loop in paragraphs
        for sentence in sentenceList:

            if nb == 0:
                STATE.fileData += STATE.date + "\t"
                if STATE.title is not None:
                    STATE.fileData += STATE.title + "\t"
                else:
                    STATE.fileData += "Unknown Book\t"

                if sentence is not None:
                    STATE.fileData += sentence.get_text() + "\t" + sentence.get_text()
                    nb += 1
            else:
                if sentence is not None:
                    STATE.fileData += " " + sentence.get_text()
                    nb += 1
                if nb == STATE.sepBy:
                    nb = 0
                    STATE.fileData += "\n"
        if STATE.fileData[-1:] != "\n":
            STATE.fileData += "\n"

    STATE.fileData = "Publication Year\tPublication Month\tPublication Day\tSource\tTitle\tAbstract\tAuthors\n" + STATE.fileData.replace("\n","\t" + STATE.authors+"\n")
    return


# Code Start

tmp.base("PDFBooktoTSV")

st.write(st.session_state.general_text_dict["description"])

GROBID_State = True
try:
    STATE.client = GrobidClient(config_path="./config.json")
except ServerUnavailableException:
    GROBID_State = False
    st.write("SERVER ERROR : GROBID server is not reacheable")

if GROBID_State:

    STATE.newFileName = st.text_input(label=STATE.general_text_dict["file_name_input"],placeholder="File.tsv")

    STATE.sepBy = st.number_input(label=STATE.general_text_dict["sep_choice"],min_value=0,max_value=20)

    st.file_uploader(st.session_state.general_text_dict['file'], type = ['pdf','zip'], key="uploadedfile")

    # checking if a file is uploaded
    if STATE.uploadedfile is not None:
         # first utilisation of errMessageLog, contain the log of every error message encountered
        STATE.errMessageLog = ""
        STATE.newFileName = checkNewFileName(STATE.newFileName, STATE.uploadedfile.name)
        if not STATE.newFileName:
            STATE.errMessageLog += st.session_state.general_text_dict['err_file_name']
            errDisplay(STATE.errMessageLog)
        else :
            if STATE.uploadedfile.type == 'application/pdf':
            
                if not os.path.exists("./book/file.pdf"):
                    with open("./book/file.pdf",'wb') as f_:
                        f_.write(STATE.uploadedfile.getbuffer())

                        requestGC("./book")

                if getXMLContent('./book/file.grobid.tei.xml'):

                    getBookTitle()

                    getDate()

                    getAuthors()

                    getFile()
            else:
                STATE.allFileData = ""
                STATE.zipDir = tempfile.TemporaryDirectory()
                os.mkdir(os.path.join(STATE.zipDir.name, "PDFtoTSV"))
                STATE.filesDir = tempfile.TemporaryDirectory()
                with ZipFile(STATE.uploadedfile, 'r') as zip:
                    for info in zip.infolist():
                        if info.filename[-4:] == ".pdf":
                            zip.extract(info, STATE.filesDir.name)
                for file in os.listdir(STATE.filesDir.name):
                    requestGC(STATE.filesDir.name)
                
                for file in os.listdir(STATE.filesDir.name):
                    if file[-15:] == ".grobid.tei.xml":
                        if getXMLContent(STATE.filesDir.name+"/"+file):
                            getBookTitle()
                            getDate()
                            getAuthors()

                            if STATE.sepBy == 0:
                                getDataPerParagraph()
                            else:
                                getDataPerNSentences()

                            with open(STATE.zipDir.name+"/PDFtoTSV/"+file[:-15]+".tsv","w") as file:
                                file.write(STATE.fileData.replace("\"","”"))

                            if STATE.allFileData == "":
                                STATE.allFileData += STATE.fileData
                            else:
                                STATE.allFileData += STATE.fileData[81:]
                            
                with open(STATE.zipDir.name+"/PDFtoTSV/compiledFiles.tsv","w") as file:
                    file.write(STATE.allFileData.replace("\"","”"))
                shutil.make_archive(STATE.filesDir.name +"/PDFtoTSV", "zip", STATE.zipDir.name)

                with open(STATE.filesDir.name +"/PDFtoTSV.zip", "rb") as zip:
                    x = zip.read()
                show_download_button(STATE.newFileName[:-3]+"zip", x)
        STATE.uploadedfile.close()
    else:
        empty_content()
